# -*- coding: utf-8 -*-
"""
Created on Tue Apr 13 11:08:54 2021

@author: Benjamin
"""

''' Ce code permet d'extraire les données issue d'un ficher txt issu du logiciel EsaoStudio.
    Il faut deux colonnes de données (position et luminance) séparées d'une tabulation.
    Données prises avec la caméra CCD OVIO. Les données sont converties dans le bon format
    et ajustées par une fonction sinuc cardinal carré. Modifier les paramètres si besoin.
    Si l'on est bien placé dans le plan de Fourier, les paramètres de l'ajustement permettent
    de remonter à la taille de l'objet diffractant que ce soit une fente ou un trou.'
'''
import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit

file = 'nom_du_fichier.txt'  #Changer le nom du fichier à ouvrir(mettre le chemin entier au besoin)
foc = 1E-3*float(file[-6:-4]) # Changer la focale selon la lentille choisie
lamb = 632E-9 # Changer la longueur d'onde selon le laser choisi


'''Définition des ajustements '''
def sinc(x,a,b,c,d):
    return a+b*np.sinc((x-d)/c)**2 # ATTENTION, sous python le sinc est défini comme sin(pi*x)/(pi*x) !! 
def bessel(x,a,b,c,d):
    return a+b*(2*sp.special.j1(np.pi*(x-d)/c)/(np.pi*(x-d)/c+1E-10))**2

''' Lecture et mise en forme du ficher .txt '''
with open(file,'r') as f:
    list1, list2 = zip(*[x.split('\t') for x in f.readlines()])
    list1 = list1[3:]
    list2 = list2[3:]
    
array1 = np.zeros(len(list1))
array2 = np.zeros(len(list2))

for i in range(len(list1)-2) :
    array1[i]=float(str.replace(list1[i],',', '.'))
for i in range(len(list2)-2) :
    array2[i]=float(str.replace(list2[i],',', '.'))

array1=sorted(array1)
array2 = array2/max(array2)
x0 = np.argmax(array2)
x0 = array1[x0]

''' Ajustement et tracé des données '''
#La boucle permet au programme de savoir la fonction avec laquelle 
#il doit ajuster, selon le titre du fichier

if file[0]=='f' :
    popt,pcov = curve_fit(sinc,array1,array2,p0=[0,1,0.001,x0])
    fit = sinc(array1, *popt);
    a=lamb*foc/(popt[2])
    print(a)
else:
    popt,pcov = curve_fit(bessel,array1,array2,p0=[0,1,0.001,x0])
    fit = bessel(array1, *popt)
    d= lamb*foc/popt[2]
    print(d)


plt.figure()
size=15
ax=plt.subplot()
ax.plot(array1,array2,'+')
ax.plot(array1, fit,'r--')
ax.set_ylabel('Intensité normalisée',fontsize=size)
ax.set_xlabel('Position sur la barette CCD (m)',fontsize=size)
ax.set_title("Figure de diffraction",fontsize=size)
plt.yticks(fontsize=size)
plt.xticks(fontsize=size)
plt.show()


 